/*
	TOYOSHIKI Tiny BASIC for Linux
	(C)2015 Tetsuya Suzuki
	Build: cc ttbasic_raspi.c basic.c -o ttbasic_raspi
*/
#include <stdlib.h>
#include <time.h>
#include <stdio.h> // for sprintf
#include <fcntl.h> // for open
#include <string.h> // for strlen

#define _LED_
#define _SW_
#define _PIO_

#ifdef _PIO_
  #ifndef _LED_
  #define _LED_
  #endif
  #ifndef _SW_
  #define _SW_
  #endif
#endif

#ifdef _SW_
const signed char sws[] = {4, 3, 2, -1};

void sw_restore(){
	int i;
	int fd;
	char buf[40];

	// GPIO unexport
	fd = open("/sys/class/gpio/unexport", O_WRONLY);
	if(fd == -1){
		perror("GPIO open");
		exit(0);
	}
	for(i = 0; sws[i] != -1; i++){
		sprintf(buf, "%d", sws[i]);
		write(fd, buf, strlen(buf));
	}
	close(fd);
}

void sw_setup(){
	int i;
	int fd;
	char buf[40];

	// GPIO export
	fd = open("/sys/class/gpio/export", O_WRONLY);
	for(i = 0; sws[i] != -1; i++){
		sprintf(buf, "%d", sws[i]);
		write(fd, buf, strlen(buf));
	}
	close(fd);

	// GPIO direction
	for(i = 0; sws[i] != -1; i++){
		sprintf(buf, "/sys/class/gpio/gpio%d/direction", sws[i]);
		fd = open(buf, O_WRONLY);
		write(fd, "high", 4);
		close(fd);
	}
}

short digitalRead(short no){
	int fd;
	char buf[40];
	char c;

	sprintf(buf, "/sys/class/gpio/gpio%d/value", no);
	fd = open(buf, O_RDONLY);
	if(fd == -1)
		return -1;
	read(fd, &c, 1);
	close(fd);
	return c - '0';
}
#endif

#ifdef _LED_
const char leds_onboard[2] = {32, 33};
const char leds_gpio[] = {};
signed char leds[64] = {-1};

void led_restore(){
	int i;
	int fd;
	char buf[40];

	fd = open("/sys/class/leds/led0/trigger", O_WRONLY);
	if(fd == -1){
		perror("ACT open");
		exit(0);
	}
	write(fd, "mmc0", 4);
	close(fd);

	fd = open("/sys/class/leds/led1/trigger", O_WRONLY);
	if(fd != -1){
		write(fd, "input", 5);
		close(fd);
	}

	// GPIO unexport
	fd = open("/sys/class/gpio/unexport", O_WRONLY);
	for(i = 0; leds[i] != -1; i++){
		sprintf(buf, "%d", leds[i]);
		write(fd, buf, strlen(buf));
	}
	close(fd);
}

void digitalWrite(short no, short on){
	int fd;
	char buf[40];

	sprintf(buf, "/sys/class/gpio/gpio%d/value", no);
	fd = open(buf, O_WRONLY);
	if(fd == -1)
		return;
	sprintf(buf, "%d", on);
	write(fd, buf, strlen(buf));
	close(fd);
}

void led_setup(){
	int i;
	int index = 0;
	int fd;
	char buf[40];

	// Try PWR setup
        fd = open("/sys/class/leds/led1/trigger", O_WRONLY);
	if(fd != -1){ // success
		write(fd, "gpio", 4);
		close(fd);

	        fd = open("/sys/class/leds/led1/gpio", O_WRONLY);
		sprintf(buf, "%d", leds_onboard[1]);
		write(fd, buf, strlen(buf));
		leds[index++] = leds_onboard[1];
		close(fd);
	}

	// ACT setup
	fd = open("/sys/class/leds/led0/trigger", O_WRONLY);
	write(fd, "gpio", 4);
	close(fd);

        fd = open("/sys/class/leds/led0/gpio", O_WRONLY);
	sprintf(buf, "%d", leds_onboard[0]);
	write(fd, buf, strlen(buf));
	leds[index++] = leds_onboard[0];
	close(fd);

	// GPIO export
	fd = open("/sys/class/gpio/export", O_WRONLY);
	for(i = 0; i < index; i++){
		sprintf(buf, "%d", leds[i]);
		write(fd, buf, strlen(buf));
	}
	for(i = 0; i < sizeof(leds_gpio); i++){
		sprintf(buf, "%d", leds_gpio[i]);
		if(write(fd, buf, strlen(buf)) != -1){
			leds[index++] = leds_gpio[i];
		}
	}
	leds[index++] = -1;
	close(fd);

	// GPIO direction
	for(i = 0; leds[i] != -1; i++){
		sprintf(buf, "/sys/class/gpio/gpio%d/direction", leds[i]);
		fd = open(buf, O_WRONLY);
		write(fd, "out", 3);
		close(fd);
	}

	digitalWrite(leds_onboard[0], 0); // ACT off
	digitalWrite(leds_onboard[1], 1); // PWR on
}
#endif

void basic(void); // prototype

int main(){
#ifdef _LED_
	led_restore();
	led_setup();
#endif
#ifdef _SW_
	sw_restore();
	sw_setup();
#endif

	srand((unsigned int)time(0)); // for RND function
	basic(); // call The BASIC

#ifdef _LED_
	led_restore();
#endif
#ifdef _SW_
	sw_restore();
#endif
	return 0;
}
